#version 330

layout (std140) uniform Matrices
{
	mat4 m_pvm;
	mat4 m_viewModel;
	mat3 m_normal;
};

layout (std140) uniform Materials
{
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
};

layout (std140) uniform Lights
{
	vec3 l_dir;		// camera space
};

in vec4 position;	// local space
in vec3 normal;		// local space

out Data
{
	/*flat */vec4 color;
} DataOut;

//----------------------------------------------------------------
// TODO:
// 1) Set the out color with computed diffuse light.
// 2) Use flat modifier for the output color.
//----------------------------------------------------------------

void main()
{
	// Transform normal to camera space and normalize it
	vec3 n = normalize(m_normal * normal);

	// Compute the intensity as the dot product
	// the max prevents negative intensity values
	float intensity = max(dot(n, l_dir), 0.f);

	// Compute the object color as diffuse color modulated by intensity
	DataOut.color = intensity * diffuse;

	gl_Position = m_pvm * position;	
}