#version 330

layout (std140) uniform Matrices
{
	mat4 m_pvm;
	mat4 m_viewModel;
	mat3 m_normal;
};

layout (std140) uniform Materials
{
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
};

layout (std140) uniform Lights
{
	vec3 l_dir;		// camera space
};

in vec4 position;	// local space
in vec3 normal;		// local space

out Data
{
	vec4 color;
} DataOut;

//----------------------------------------------------------------
// TODO:
// 1) Compute directional lighting using ambient material color + computed diffuse light.
// 2) Prevent out-of-range color values.
//----------------------------------------------------------------


void main()
{
	vec3 n = normalize(m_normal * normal);

	float intensity = max(dot(n, l_dir), 0.f);

	// Compute the color as the sum of the two components
	vec4 col = ambient + intensity * diffuse;
	// Clamp the color to prevent out-of-range values
	DataOut.color = clamp(col, 0.f, 1.f);

	gl_Position = m_pvm * position;	
}