#version 330

layout (std140) uniform Matrices
{
	mat4 m_pvm;
	mat4 m_viewModel;
	mat3 m_normal;
};

layout (std140) uniform Materials
{
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
};

layout (std140) uniform Lights
{
	vec3 l_dir;		// camera space
};

in vec4 position;	// local space
in vec3 normal;		// local space

out Data
{
	vec4 color;
} DataOut;

//----------------------------------------------------------------
// TODO:
// 1) Compute directional lighting using ambient component + diffuse component + the specular component.
//----------------------------------------------------------------

void main()
{
	vec4 spec = vec4(0.f);

	vec3 n = normalize(m_normal * normal);

	float intensity = max(dot(n, l_dir), 0.f);

	if(intensity > 0.f)
	{
		vec3 pos = vec3(m_viewModel * position);
		vec3 eye = normalize(-pos);
		vec3 h = normalize(l_dir + eye);

		float intSpec = max(dot(h, n), 0.f);
		spec = specular * pow(intSpec, shininess);
	}
	
	// Compute the color as the sum of the three components
	vec4 col = ambient + intensity * diffuse + spec;
	DataOut.color = clamp(col, 0.f, 1.f);

	gl_Position = m_pvm * position;	
}