#version 330

// 0 Phong
// 1 Blinn-Phong
#define TYPE 0

layout (std140) uniform Matrices
{
	mat4 m_pvm;
	mat4 m_viewModel;
	mat3 m_normal;
};

layout (std140) uniform Materials
{
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
};

layout (std140) uniform Lights
{
	vec3 l_dir;		// camera space
};

in vec4 position;	// local space
in vec3 normal;		// local space

out Data
{
	vec4 color;
} DataOut;

//----------------------------------------------------------------
// TODO:
// 1) Set the out color with computed specular light using the Phong model.
//	- Eye vector can easily be computed in camera space: convert position to camera space and compute vector from this position to the origin.
// 2) Set the out color with computed specular light using the Blinn-Phong model.
//----------------------------------------------------------------

void main()
{
	// Set the specular term to black
	vec4 spec = vec4(0.f);

	vec3 n = normalize(m_normal * normal);

	float intensity = max(dot(n, l_dir), 0.f);

	// If the vertex is lit, compute the specular color
	if(intensity > 0.f)
	{
		// Compute vertex position in camera space
		vec3 pos = vec3(m_viewModel * position);
		// Compute eye vector in camera space and normalize it 
		vec3 eye = normalize(-pos);

#if TYPE == 0
		// We need: R = -L + 2.0 * dot(N, L) * N 
		// reflect: R = I - 2.0 * dot(N, I) * N where I = L
		vec3 r = -normalize(reflect(l_dir, n));

		// Compute the specular term as the angle between the reflection vector and the eye vector
		float intSpec = max(dot(r, eye), 0.f);
#else
		// Compute the half vector
		vec3 h = normalize(l_dir + eye);

		// Compute the specular term as the angle between the normal and the half vector
		float intSpec = max(dot(h, n), 0.f);
#endif

		spec = specular * pow(intSpec, shininess);
	}
	
	// Use specular color when the vertex is lit
	DataOut.color = spec;

	gl_Position = m_pvm * position;	
}