#version 330

layout (std140) uniform Materials
{
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
};

in Data
{
	vec3 normal;	// camera space
	vec3 eye;		// camera space
	vec3 lightDir;	// camera space
} DataIn;

out vec4 outputF;

//----------------------------------------------------------------
// TODO:
// 1) Compute point lighting using ambient component + diffuse component + the specular component using the interpolated values.
//----------------------------------------------------------------

void main()
{
	vec4 spec = vec4(0.f);

	vec3 n = normalize(DataIn.normal);
	vec3 l = normalize(DataIn.lightDir);
	vec3 e = normalize(DataIn.eye);

	float intensity = max(dot(n, l), 0.f);

	if(intensity > 0.f)
	{
		vec3 h = normalize(l + e);
		float intSpec = max(dot(h, n), 0.f);
		spec = specular * pow(intSpec, shininess);
	}
	
	vec4 col = ambient + intensity * diffuse + spec;
	outputF = clamp(col, 0.f, 1.f);
}