#version 330

layout (std140) uniform Materials
{
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
};

layout (std140) uniform Lights
{
	vec4 l_pos, l_spotDir;		// camera space
	float l_spotCutOff;			// cosinus of angle
};

in Data
{
	vec3 normal;	// camera space
	vec3 eye;		// camera space
	vec3 lightDir;	// camera space
} DataIn;

out vec4 outputF;

//----------------------------------------------------------------
// TODO:
// 1) Compute point lighting using ambient component + diffuse component + the specular component using the interpolated values.
//		- Prevent any light falling outside the l_spotCutOff angle.
//----------------------------------------------------------------

void main()
{
	float intensity = 0.f;
	vec4 spec = vec4(0.f);

	vec3 l = normalize(DataIn.lightDir);
	vec3 s = normalize(vec3(-l_spotDir));

	// If the pixel is inside the light cone
	if(dot(s, l) > l_spotCutOff)
	{
		vec3 n = normalize(DataIn.normal);
		intensity = max(dot(n, l), 0.f);
		
		if(intensity > 0.f)
		{
			vec3 e = normalize(DataIn.eye);
			vec3 h = normalize(l + e);
			float intSpec = max(dot(h, n), 0.f);
			spec = specular * pow(intSpec, shininess);
		}
	}
	
	vec4 col = ambient + intensity * diffuse + spec;
	outputF = clamp(col, 0.f, 1.f);
}